@@ -268,10 +268,12 @@ class DataLoader:
         unique_products_df["productID"] = ids
         
         return products, unique_products_df
-    
+     
     def write_product_categories(self, products: pd.DataFrame, unique_products_df: pd.DataFrame, company_object: Company, category_object: Categories, product_object: Products) -> t.Any:
         
-        """Link products to a category and write associations.
+        """
+        Link products to a category and write associations.
+        
         """
 
         products = clean_and_check_products(products, copy=True)
@@ -285,6 +287,10 @@ class DataLoader:
         products_with_ids_and_category_ids = products_with_ids_and_category_ids[["productID", "categoryID"]]
 
         product_categories = ProductCategories(self.db, self.insertion_mode)
+        
+        if products_with_ids_and_category_ids.isnull().values.any():
+            raise ValueError("There are NaN values in the product-category mapping. Ensure all products and categories exist in the database.")
+
         product_categories.write_to_db_multi_row(products_with_ids_and_category_ids, save_ids=False, show_progress_bar=True)
 
         return product_categories
@@ -302,17 +308,11 @@ class DataLoader:
         store_ids_df = get_data_ids_by_company(self.db, company_id=company_object.ids, table_name = "stores", column_name="name", datapoints=unique_stores, output_column_name="store", output_column_ID="storeID")
         product_ids_df = get_data_ids_by_company(self.db, company_id=company_object.ids, table_name = "products",  column_name="name", datapoints=unique_products, output_column_name="product", output_column_ID="productID")
 
-        print(product_ids_df)
-        print(store_ids_df)
-
         time_sku_data = time_sku_data.merge(store_ids_df, left_on="store", right_on="store", how="left")
         time_sku_data = time_sku_data.merge(product_ids_df, left_on="product", right_on="product", how="left")
 
         # get unique store product pairs
         unique_store_product_pairs = time_sku_data[["storeID", "productID"]].drop_duplicates()
-        print(unique_store_product_pairs)
-
-        assert False
 
         skus = SKUTable(self.db, self.insertion_mode)
         skus.write_to_db_multi_row(unique_store_product_pairs, save_ids=True, show_progress_bar=True)
@@ -409,6 +409,7 @@ class DataLoader:
         time_sku_feature_map = time_sku_data_with_datapoint_id.merge(time_sku_feature_description_map, on="feature", how="left")
 
         time_sku_feature_map["featureID"] = time_sku_feature_map["featureID"].astype("int")
+        time_sku_feature_map["value"] = time_sku_feature_map["value"].astype("float")
         del time_sku_feature_map["feature"]
 
         logger.info("Writing time-sku features to the database")
@@ -470,18 +471,10 @@ class DataLoader:
         args = list(zip(flags["product"], flags["date"].astype("string"), flags["store"], [company_id] * len(flags)))
         args = tuple(args)
 
-        print("query:", query)
-
-        # print first 10 args
-        print(f"Args for retrieving datapoint IDs: {args[:10]}")
-
         logger.info("Retrieving datapoint IDs for flags ...")
         # TODO: change this to get a one-shot query instead of line-by-line.
         datapoint_ids = self.db.execute_multi_query(query, args, fetchall=True, show_progress_bar=True)
 
-        print("Datapoint IDs retrieved:")
-        print(datapoint_ids[:10])  # Print first 10 datapoint IDs for debugging
-
         # check if datapoint_ids are empty
         if datapoint_ids:
             datapoint_ids = [datapoint_id[0] if len(datapoint_id) > 0 else datapoint_id for datapoint_id in datapoint_ids]
@@ -503,6 +496,8 @@ class DataLoader:
         del flags["feature"]
         del flags["feature_type"]
 
+        flags["value"] = flags["value"].astype("float")
+
         time_sku_features_object = TimeSkuFeatures(self.db, self.insertion_mode)
         time_sku_features_object.write_to_db_multi_row(flags, save_ids=False, show_progress_bar=True)
 
